# FB_Cusum_Chisq_Simu.R
# EU project Fisboat 502572
# Pierre Petitgas, Ifremer
# 16 July 2007
#
# Estimation of the ARL for an upward shift in the non centrality parameter of 
# a chi-square process using a DI-Cusum scheme: Chi2(df,0) -> shifting to Chi2(df,ncp)
# estimation by simulations using the variance reduction technique of 
#   Jun and Choi. 1993. Commun. Statist. Simula. 22: 877-887
# a first choice for ck is suggested based on Gamma cusum scheme (Hawkins and Olwell, 1998, chap6) 
# Chisq(df,0)=G(df/2,2) -> shifting to G(df/2,b1) with b1=2+2*ncp/df 
# b1=2+2*ncp/df obtained by equating E[G(df/2,b1)]=E[Chisq(df,ncp)]


#### FUNCTIONS

dicusum1.f <- function(data,cck) {
# Computes cusum phi+ 
#  data = Raw-data vector ; cck = allowance k
 nd <- length(data)+1; phip <- vector(mode="numeric",length=nd)
 phip[1] <- 0
 for(i in 2:nd) {phip[i] <- max(0,(phip[i-1]+data[i-1]-cck),na.rm=TRUE)}
 return(phip[2:nd])
}

ck.chisq.f<-function(ddf,nncp) {
# optimal ck (tolerance) for the di-cusum scheme for shift in the scale parameter of a Gamma
# Hawkins & Olwell. 1998. Springer book.  
 a0<-ddf/2; b0<-2; a1<-ddf/2; b1<-2+2*nncp/ddf
 cck<- -a0*log(b1/b0)/(1/b1-1/b0);return(cck) }

arl.chisq.f<-function(ddf,nncp,cck,cch) {
# estimation by simulation using a variance reduction technique 
# with the Cusum total hazard as control variate
# Jun and Choi. 1993. Commun. Statist. Simula. 22: 877-887
# mean(rl) using nr=5000 realisations of rl
# var(arl)=mean[(rl-arl)2] estimated by boostrap 
# using 100 resampled rl values with 100 repetitions  
 nr<-5000; nn<-rep(0,nr);yy<-rep(0,nr)
 nboot<-100; d1boot<-rep(0,nboot)
 for (k in 1:nr) {
 s<-0;i<-0;p<-0
 while(s<cch) {i<-i+1
              p<-p+1-pchisq(cck+cch-s,ddf,nncp) 
              x<-rchisq(1,ddf,nncp) 
              s<-max(0,s+x-cck) }
 nn[k]<-i; yy[k]<-p }
 dd1<-mean(nn)-cov(nn,yy)*(mean(yy)-1)/var(yy)
 qq25<-quantile(nn,0.25);  qq75<-quantile(nn,0.75)
 for (k in 1:nboot) {
  ii<-sample(1:nr, size=100, replace=T, prob = NULL)
  d1boot[k]<-mean(nn[ii])-cov(nn[ii],yy[ii])*(mean(yy[ii])-1)/var(yy[ii]) }
 ssd1<-sqrt(var(d1boot))
 x11(); hist(nn,freq=F,nclas=100,main='',xlab='Run Length')
 cat("ARL: ",round(dd1,2), "; sd: ", round(ssd1,2),"; q25: ",round(qq25,2) ,"; q75: ",round(qq75,2) ,"\n")
 res<-c(ic.arl=dd1,sd=ssd1,q25=qq25,q75=qq75)
 return(res)
}


#### COMMANDS
pat<-"C:/ptg/WORK/fp6_fisboat/wksp_R/R_multivariate_control/codNS_d2spa_mfa/"
fic<-"codNS_mfa_index.txt"
d2<-read.table(paste(pat,fic,sep=""),header=T,sep="\t")
x11();plot(d2[,1],d2[,2],type="b",pch=18,xlab='',ylab='d^2 spatial')

df<-10                                # in control : mean[chi2(df,0)]=df
seldev<-c(1995,2001,2003,2004) #1998:2002  # give years out of control
ncp<-mean(d2[is.element(d2[,1],seldev),2])-df       # ncp<-11 
ck<-ck.chisq.f(df,ncp)                             
ch<-5  
cat("ncp: ",round(ncp,2),"; ck: ",round(ck,2),"; ch: ",round(ch,2),"\n")

phi<-dicusum1.f(d2[,2],ck)
x11();plot(d2[,1],phi,type="b",xlab='',ylab='DI-Cusum',cex.lab=1.1)
      abline(h=ch,lty=2)

ic.arl<-arl.chisq.f(df,0,ck,ch)
oc.arl<-arl.chisq.f(df,ncp,ck,ch)

mfachisq.res<-list(df=df,ncp=ncp,k=ck,h=ch,ic.arl=ic.arl,oc.arl=oc.arl)
assign(paste("mfachisq.",substring(fic,1,5),sep=""),mfachisq.res)

